@@ -15,6 +15,9 @@ class OpisClosureSerializer implements ClosureSerializer {
      */
     public function serialize(Closure $closure)
     {
+        if (function_exists('Opis\Closure\serialize')) {
+            return \Opis\Closure\serialize($closure);
+        }
         return serialize(new OpisSerializableClosure($closure));
     }
 
@@ -26,6 +29,9 @@ class OpisClosureSerializer implements ClosureSerializer {
      */
     public function unserialize($serialized)
     {
+        if (function_exists('Opis\Closure\unserialize')) {
+            return \Opis\Closure\unserialize($serialized);
+        }
         return unserialize($serialized)->getClosure();
     }
 
